//////////////////////////////////////////////////////////////////////////////// 
/// Plainamp, Open source Winamp core
/// 
/// Copyright  2005  Sebastian Pipping <webmaster@hartwork.org>
/// 
/// -->  http://www.hartwork.org
/// 
/// This source code is released under the GNU General Public License (GPL).
/// See GPL.txt for details. Any non-GPL usage is strictly forbidden.
////////////////////////////////////////////////////////////////////////////////


#include "Output.h"
#include "OutputPlugin.h"
#include "Console.h"
#include <time.h>



int  Output_Open( int samplerate, int numchannels, int bitspersamp, int bufferlenms, int prebufferms );
void Output_Close();
int  Output_Write( char * buf, int len );
int  Output_CanWrite();
int  Output_IsPlaying();
int  Output_Pause( int pause );
void Output_SetVolume( int volume );
void Output_SetPan( int pan );
void Output_Flush( int t );
int  Output_GetOutputTime();
int  Output_GetWrittenTime();



////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
Out_Module output_server = {
	OUT_VER,                        // int version
	"Plainamp Output Server",       // char * description;
	0x7fffffff,                     // int id
	NULL,                           // HWND hMainWindow
	NULL,                           // HINSTANCE hDllInstance
	NULL,                           // void (*Config)(HWND hwndParent);
	NULL,                           // void (*About)(HWND hwndParent);
	NULL,                           // void (*Init)();
	NULL,                           // void (*Quit)();
	Output_Open,                    // int (*Open)(int samplerate, int numchannels, int bitspersamp, int bufferlenms, int prebufferms); 
	Output_Close,                   // void (*Close)();
	Output_Write,                   // int (*Write)(char *buf, int len);
	Output_CanWrite,                // int (*CanWrite)();
	Output_IsPlaying,               // int (*IsPlaying)();
	Output_Pause,                   // int (*Pause)(int pause);
	Output_SetVolume,               // void (*SetVolume)(int volume);
	Output_SetPan,                  // void (*SetPan)(int pan);
	Output_Flush,                   // void (*Flush)(int t);
	Output_GetOutputTime,           // int (*GetOutputTime)();
	Output_GetWrittenTime,          // int (*GetWrittenTime)();
};



////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
int Output_Open( int samplerate, int numchannels, int bitspersamp, int bufferlenms, int prebufferms )
{
	// Maximum
	int res = 0;
	int res_temp;
	for( int i = 0; i < active_output_count; i++ )
	{
		res_temp = active_output_plugins[ i ]->plugin->Open( samplerate, numchannels, bitspersamp, bufferlenms, prebufferms );
		if( res_temp > res ) res = res_temp;
	}
	return res;
}



////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
void Output_Close()
{
	for( int i = 0; i < active_output_count; i++ )
	{
		active_output_plugins[ i ]->plugin->Close();
	}
}



////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
int Output_Write( char * buf, int len )
{
	// Maximum
	int res = 0;
	int res_temp;
	for( int i = 0; i < active_output_count; i++ )
	{
		res_temp = active_output_plugins[ i ]->plugin->Write( buf, len );
		if( res_temp > res ) res = res_temp;
	}
	return res;
}



////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
int Output_CanWrite()
{
	// Minimum
	int res = 0x7fffffff;
	int res_temp;
	for( int i = 0; i < active_output_count; i++ )
	{
		res_temp = active_output_plugins[ i ]->plugin->CanWrite();
		if( res_temp < res ) res = res_temp;
	}
	return res;
}



////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
int Output_IsPlaying()
{
	// Maximum	
	int res = 0;
	int res_temp;
	for( int i = 0; i < active_output_count; i++ )
	{
		res_temp = active_output_plugins[ i ]->plugin->IsPlaying();
		if( res_temp > res ) res = res_temp;
	}
	return res;
}



////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
int Output_Pause( int pause )
{
	// Maximum
	int res = 0;
	int res_temp;
	for( int i = 0; i < active_output_count; i++ )
	{
		res_temp = active_output_plugins[ i ]->plugin->Pause( pause );
		if( res_temp > res ) res = res_temp;
	}
	return res;
}



////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
void Output_SetVolume( int volume )
{
	for( int i = 0; i < active_output_count; i++ )
	{
		active_output_plugins[ i ]->plugin->SetVolume( volume );
	}
}



////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
void Output_SetPan( int pan )
{
	for( int i = 0; i < active_output_count; i++ )
	{
		active_output_plugins[ i ]->plugin->SetPan( pan );
	}
}



////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
void Output_Flush( int t )
{
	for( int i = 0; i < active_output_count; i++ )
	{
		active_output_plugins[ i ]->plugin->Flush( t );
	}
}



////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
int Output_GetOutputTime() // <= GetWrittenTime()
{
	// Minimum
	int res = 0x7fffffff;
	int res_temp;
	for( int i = 0; i < active_output_count; i++ )
	{
		res_temp = active_output_plugins[ i ]->plugin->GetOutputTime();
		if( res_temp < res ) res = res_temp;
	}
	return res;
}



////////////////////////////////////////////////////////////////////////////////
///
////////////////////////////////////////////////////////////////////////////////
int Output_GetWrittenTime()
{
	// Maximum
	int res = 0;
	int res_temp;
	for( int i = 0; i < active_output_count; i++ )
	{
		res_temp = active_output_plugins[ i ]->plugin->GetWrittenTime();
		if( res_temp > res ) res = res_temp;
	}
	return res;
}
